<?php
/* 
*   +------------------------------------------------------------------------------+ 
*       BrewThology - Online Recipe Database with calculator and BeerXML support
*		http://www.brewthology.com
*		includes\formulas5.inc - PHP5 Functions to calculate recipe statistics
*		version: 0.1
*   +------------------------------------------------------------------------------+ 
*    Copyright: This program is free software; you can redistribute it and/or modify
*    it under the terms of the license included with the package.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
*   +------------------------------------------------------------------------------+ 
*       Author: 
*			Rob Hudson (tastybrew.com)
*		Contributors:	
*			Bryan Peretto (kotmf.com - bperetto@yahoo.com)
*			Mike A. (barleydogbrewery.com)
*   +------------------------------------------------------------------------------+ 
*/ 

class Recipe
{



  // TODO: Add error checking
  public $id;

  // CONSTANTS
  public $steep_yield = .33;
  public $pellet_increase = 1.10;
  // publicious other hopping methods
  // A xxx addition is similar to a yyy addition in terms of bitterness...
  public $MH_factor = 10;
  public $FWH_factor = 20;
  public $DH_factor = 0;

  public $efficiency;
  public $boil_vol;
  public $batch_vol;
  
  public $specs  = array();
  public $grains = array();
  public $hops   = array();
  public $misc   = array();

  public $grain_total_wt;
  public $hop_total_wt;

  public $_OG;  // Original gravity
  public $_FG;  // Final gravity
  public $_SRM; // Standard Reference Material (color)
  public $_IBU; // International Bittering Unit
  public $_ABV; // Alcohol by Volume
  public $_ABW; // Alcohol by Weight

  // Configuration
  public $units = 'US'; //public $units = 'metric';

  // Error array
  public $error = array();


  // Constructor
  function Recipe()
  {
  }

  function calc_all()
  {
    $this->calc_og();
    $this->calc_fg();
    if ($this->batch_vol > $this->boil_vol)
    {
      $this->calc_ibu_rager();
    }
    else
    {
      $this->calc_ibu_tinseth();
    }
    $this->calc_srm();
    $this->calc_abv();
    $this->calc_abw();
  }

  function set_efficiency($eff)
  {
    if ($eff > 0 && $eff <= 1)
    {
      $this->efficiency = $eff;
    }
  }

  function set_batch_vol($vol)
  {
    if ($vol > 0)
    {
      $this->batch_vol = $vol;
    }
  }

  function set_boil_vol($vol)
  {
    if ($vol > 0)
    {
      $this->boil_vol = $vol;
    }
  }

  function set_units($units)
  {
    if ($units == 'US' || $units == 'metric')
    {
      $this->units = $units;
    }
  }

  // Assumes extract = ppg (eg: 2-row = 37 ppg)
  function add_grain($name, $weight, $note, $extract, $color, $usage)
  {
    $yield = $weight * ($extract / 1000);
    $hcu = $weight * $color;

    $this->grains[] = 
      array(
        'weight'  => $weight, 
        'name'    => $name, 
        'note'    => $note,
        'usage'   => $usage,
        'extract' => $extract, 
        'color'   => $color, 
        'yield'   => $yield,
        'hcu'     => $hcu
      );
  }

  function add_hop($name, $note, $weight, $form, $alpha, $time)
  {
    $hbu = $weight * $alpha;

/*
    switch($time)
    {
      case 'MH':
      case 'FWH':
      case 'DH':
    }
*/

    $this->hops[] =
      array(
        'name'   => $name,
        'note'   => $note,
        'weight' => $weight,
        'form'   => $form,
        'alpha'  => $alpha,
        'time'   => $time,
        'hbu'    => $hbu
      );
  }

  function add_yeast($name, $attenuation)
  {
    $this->specs['yeast'] = $name;
    if ($attenuation > 0 && $attenuation < 100)
    {
      $this->specs['attenuation'] = $attenuation;
    }
    else
    {
      $this->error[] = "Incorrect attenuation value.";
    }
  }

  function add_misc($amount, $name, $note)
  {
    $this->misc[] =
      array(
        'amount' => $amount,
        'name'   => $name,
        'note'   => $note
      );
  }

  function calc_og()
  {
    if (count($this->grains) > 0)
    {
      $est = 0.0;
      $total_wt = 0;

      foreach ($this->grains as $idx => $grain)
      {
        $yield = $grain['yield'];

        switch ($grain['usage'])
        {
          case 'Mashed':
            $yield *= $this->efficiency;
            break;
          case 'Steeped':
            $yield *= $this->steep_yield;
            break;
          case 'Extract':
            break; // extract yield is 100%
        } // end switch

        $est += $yield;
        $this->grains[$idx]['grav_points'] = ($yield / $this->batch_vol) * 1000;
        $total_wt += $grain['weight'];

      } // end foreach

      $est /= $this->batch_vol;
      $this->grain_total_wt = $total_wt;
      $this->_OG = number_format($est + 1.0, 3);
    }
  }

  // Rager is more accurate for concentrated boils
  function calc_ibu_rager()
  {
	$boil_grav = 1 + ( ($this->_OG - 1) * ( $this->batch_vol / $this->boil_vol ) );

    if (count($this->hops) > 0)
    {
      $bitterness = 0.0;
      $total_wt = 0.0;

      foreach ($this->hops as $idx => $hop)
      {
        $total_wt += $hop['weight'];
        switch($hop['time'])
        {
          case 'MH':
            $time = $this->MH_factor;
            break;
          case 'FWH':
            $time = $this->FWH_factor;
            break;
          case 'DH';
            $time = $this->DH_factor;
            break;
          default:
            $time = $hop['time'];
        }

        $util = $this->rager_utilization($time);
        if ($hop['form'] == 'Pellet')
        {
          $util *= $this->pellet_increase;
        }

        // correct for boil gravity
        $grav_adj = 1;
        if ($boil_grav > 1.050)
        {
          $grav_adj += (($boil_grav - 1.050) / 0.2);
          $util /= ($this->boil_vol * $grav_adj);
        }

        // correct for concentrated boil
        if ($this->batch_vol > $this->boil_vol)
        {
          $util *= ($this->boil_vol / $this->batch_vol);
        }

        $isoalpha = ( $hop['hbu'] * $util );

        $this->hops[$idx]['isoalpha'] = $isoalpha;
        $this->hops[$idx]['utilization'] = $util / 100;

        $bitterness += $isoalpha;
      }

      $this->hop_total_wt = $total_wt;
      $this->_IBU = number_format($bitterness,1);
    }
  }

  function rager_utilization($time)
  {
    $util = array(
      '90' => 31,
      '80' => 30,
      '70' => 29,
      '60' => 28,
      '45' => 26,
      '40' => 22,
      '35' => 18,
      '30' => 15,
      '25' => 12,
      '20' => 10,
      '15' => 8,
      '10' => 6,
      '05' => 5,
      '01' => 1,
      '00' => 0
    );
    foreach ($util as $_time => $_val)
    {
      if ($time >= $_time)
      {
        return $_val;
      }
    }
    return 0.0;
  }

  // Tinseth is more accurate for full boils
  function calc_ibu_tinseth()
  {
	// Determine approx boil gravity
    $boil_grav = ( ($this->_OG - 1) * ( $this->batch_vol / $this->boil_vol ) );

    if (count($this->hops) > 0)
    {
      $bitterness = 0.0;
      $total_wt = 0;

      foreach ($this->hops as $idx => $hop)
      {
        switch($hop['time'])
        {
          case 'MH':
            $time = $this->MH_factor;
            break;
          case 'FWH':
            $time = $this->FWH_factor;
            break;
          case 'DH';
            $time = $this->DH_factor;
            break;
          default:
            $time = $hop['time'];
        }
        $total_wt   += $hop['weight'];
        $grav_factor = 1.65 * pow(0.000125, $boil_grav);
        $time_factor = ( 1 - exp(-0.04 * $time) ) / 4.15;
        $util        = $grav_factor * $time_factor;
        if ($hop['form'] == 'Pellet')
        {
          $util *= $this->pellet_increase;
        }
        $isoalpha    = ( $hop['hbu'] * $util * 74.89 ) / $this->batch_vol;

        $this->hops[$idx]['utilization'] = $util;
        $this->hops[$idx]['isoalpha'] = $isoalpha;

        $bitterness += $isoalpha;
      }
      //bitterness *= (GPO / LPG) / 100.0;
      $this->hop_total_wt = $total_wt;
      $this->_IBU = number_format($bitterness,1);
    }
  }

  function calc_srm()
  {
    if (count($this->grains) > 0)
    {
      $srm = 0.0;
      foreach ($this->grains as $idx => $grain)
      {
        //$color = $this->efficiency * $grain['hcu'];
        $color = $grain['hcu'];
        $srm += $color;
        $this->grains[$idx]['color_points'] = $color / $this->batch_vol;
      }
      $srm /= $this->batch_vol;

      // Use Dan A. Morey's formula
      if ($srm > 8.0) 
      {
          $srm = (pow($srm,0.6859)) * 1.4922;
          if ($srm > 50)
          {
            $srm = 50;
          }
      }
      $this->_SRM = round($srm);
    }
  }

  function calc_fg()
  {
    if (isset($this->_FG))
    {
      return;
    }
    if ($this->_OG <= 0.0)
    {
      return 0.0;
    }
    $fg = ((($this->_OG - 1.0) * (1.0 - ($this->specs['attenuation'] / 100) )) + 1.0);
    $this->_FG = number_format($fg,3);
  }

  function calc_abv()
  {
    $this->_ABV = number_format(((($this->_OG - $this->_FG) * 1.29) * 100), 1); 
  }

  function calc_abw()
  {
    $this->_ABW = number_format(((($this->_ABV / 100) * 0.785) * 100), 1); 
  }




} // end class


?>
